@@ -5,7 +5,8 @@ import os
 from ceph_deploy import hosts
 from ceph_deploy.cliutil import priority
 from ceph_deploy.lib import remoto
-
+from ceph_deploy.util.constants import default_components
+from ceph_deploy.util.paths import gpg
 
 LOG = logging.getLogger(__name__)
 
@@ -16,7 +17,7 @@ def sanitize_args(args):
     not well suited for.
     """
     if args.release is None:
-        args.release = 'giant'
+        args.release = 'hammer'
         args.default_release = True
 
     # XXX This whole dance is because --stable is getting deprecated
@@ -28,8 +29,58 @@ def sanitize_args(args):
     return args
 
 
+def detect_components(args, distro):
+    """
+    Since the package split, now there are various different ceph components to
+    install like:
+
+    * ceph
+    * ceph-mon
+    * ceph-osd
+    * ceph-mds
+
+    This helper function should parse the args that may contain specifics about
+    these flags and return the default if none are passed in (which is, install
+    everything)
+    """
+    # the flag that prevents all logic here is the `--repo` flag which is used
+    # when no packages should be installed, just the repo files, so check for
+    # that here and return an empty list (which is equivalent to say 'no
+    # packages should be installed')
+    if args.repo:
+        return []
+
+    flags = {
+        'install_osd': 'ceph-osd',
+        'install_rgw': 'ceph-radosgw',
+        'install_mds': 'ceph-mds',
+        'install_mon': 'ceph-mon',
+        'install_common': 'ceph-common',
+    }
+
+    if distro.is_rpm:
+        defaults = default_components.rpm
+    else:
+        defaults = default_components.deb
+        # different naming convention for deb than rpm for radosgw
+        flags['install_rgw'] = 'radosgw'
+
+    if args.install_all:
+        return defaults
+    else:
+        components = []
+        for k, v in flags.items():
+            if getattr(args, k, False):
+                components.append(v)
+        # if we have some components selected from flags then return that,
+        # otherwise return defaults because no flags and no `--repo` means we
+        # should get all of them by default
+        return components or defaults
+
+
 def install(args):
     args = sanitize_args(args)
+
     if args.repo:
         return install_repo(args)
 
@@ -58,8 +109,8 @@ def install(args):
             # upstream. If default_release is True, it means that the user is
             # trying to install on a RHEL machine and should expect to get RHEL
             # packages. Otherwise, it will need to specify either a specific
-            # version, or repo, or a development branch. Other distro users should
-            # not see any differences.
+            # version, or repo, or a development branch. Other distro users
+            # should not see any differences.
             use_rhceph=args.default_release,
             )
         LOG.info(
@@ -69,6 +120,7 @@ def install(args):
             distro.codename
         )
 
+        components = detect_components(args, distro)
         if distro.init == 'sysvinit' and args.cluster != 'ceph':
             LOG.error('refusing to install on host: %s, with custom cluster name: %s' % (
                     hostname,
@@ -86,7 +138,7 @@ def install(args):
         # custom repo arguments
         repo_url = os.environ.get('CEPH_DEPLOY_REPO_URL') or args.repo_url
         gpg_url = os.environ.get('CEPH_DEPLOY_GPG_URL') or args.gpg_url
-        gpg_fallback = 'https://ceph.com/git/?p=ceph.git;a=blob_plain;f=keys/release.asc'
+        gpg_fallback = gpg.url('release')
 
         if gpg_url is None and repo_url:
             LOG.warning('--gpg-url was not used, will fallback')
@@ -114,7 +166,8 @@ def install(args):
                 distro,
                 repo_url,
                 gpg_url,
-                args.adjust_repos
+                args.adjust_repos,
+                components=components,
             )
 
         # Detect and install custom repos here if needed
@@ -127,7 +180,8 @@ def install(args):
                 distro,
                 args.version_kind,
                 version,
-                args.adjust_repos
+                args.adjust_repos,
+                components=components,
             )
 
         # Check the ceph version we just installed
@@ -162,6 +216,7 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
     used.
     """
     default_repo = cd_conf.get_default_repo()
+    components = detect_components(args, distro)
     if args.release in cd_conf.get_repos():
         LOG.info('will use repository from conf: %s' % args.release)
         default_repo = args.release
@@ -184,6 +239,7 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
                 default_repo,
                 options.pop('baseurl'),
                 options.pop('gpgkey'),
+                components=components,
                 **options
             )
         except KeyError as err:
@@ -198,6 +254,7 @@ def custom_repo(distro, args, cd_conf, rlogger, install_ceph=None):
                     xrepo,
                     options.pop('baseurl'),
                     options.pop('gpgkey'),
+                    components=components,
                     **options
                 )
             except KeyError as err:
@@ -233,7 +290,7 @@ def install_repo(args):
             distro.codename
         )
 
-        return custom_repo(distro, args, cd_conf, rlogger, install_ceph=False)
+        custom_repo(distro, args, cd_conf, rlogger, install_ceph=False)
 
 
 def uninstall(args):
@@ -255,7 +312,7 @@ def uninstall(args):
         LOG.info('Distro info: %s %s %s', distro.name, distro.release, distro.codename)
         rlogger = logging.getLogger(hostname)
         rlogger.info('uninstalling ceph on %s' % hostname)
-        distro.uninstall(distro.conn)
+        distro.uninstall(distro)
         distro.conn.exit()
 
 
@@ -280,7 +337,7 @@ def purge(args):
         LOG.info('Distro info: %s %s %s', distro.name, distro.release, distro.codename)
         rlogger = logging.getLogger(hostname)
         rlogger.info('purging host ... %s' % hostname)
-        distro.uninstall(distro.conn, purge=True)
+        distro.uninstall(distro, purge=True)
         distro.conn.exit()
 
 
@@ -422,34 +479,80 @@ def make(parser):
                 or tag (default: %(default)s)',
     )
 
-    version.add_argument(
+    version.set_defaults(
+        stable=None,  # XXX deprecated in favor of release
+        release=None,  # Set the default release in sanitize_args()
+        dev='master',
+        version_kind='stable',
+    )
+
+    parser.add_argument(
+        '--mon',
+        dest='install_mon',
+        action='store_true',
+        help='install the mon component only',
+    )
+
+    parser.add_argument(
+        '--mds',
+        dest='install_mds',
+        action='store_true',
+        help='install the mds component only',
+    )
+
+    parser.add_argument(
+        '--rgw',
+        dest='install_rgw',
+        action='store_true',
+        help='install the rgw component only',
+    )
+
+    parser.add_argument(
+        '--osd',
+        dest='install_osd',
+        action='store_true',
+        help='install the osd component only',
+    )
+
+    parser.add_argument(
+        '--cli', '--common',
+        dest='install_common',
+        action='store_true',
+        help='install the common component only',
+    )
+
+    parser.add_argument(
+        '--all',
+        dest='install_all',
+        action='store_true',
+        help='install all ceph components (e.g. mon,osd,mds,rgw). This is the default',
+    )
+
+    repo = parser.add_mutually_exclusive_group()
+
+    repo.add_argument(
         '--adjust-repos',
         dest='adjust_repos',
         action='store_true',
         help='install packages modifying source repos',
     )
 
-    version.add_argument(
+    repo.add_argument(
         '--no-adjust-repos',
         dest='adjust_repos',
         action='store_false',
         help='install packages without modifying source repos',
     )
 
-    version.set_defaults(
-        func=install,
-        stable=None,  # XXX deprecated in favor of release
-        release=None,  # Set the default release in sanitize_args()
-        dev='master',
-        version_kind='stable',
-        adjust_repos=True,
-    )
-
-    parser.add_argument(
+    repo.add_argument(
         '--repo',
         action='store_true',
         help='install repo files only (skips package installation)',
-        )
+    )
+
+    repo.set_defaults(
+        adjust_repos=True,
+    )
 
     parser.add_argument(
         'host',
